package com.xtd.project.assets.service.impl;

import java.util.List;
import com.xtd.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.xtd.project.assets.mapper.DbTodoItemMapper;
import com.xtd.project.assets.domain.DbTodoItem;
import com.xtd.project.assets.service.IDbTodoItemService;

/**
 * 待办事项Service业务层处理
 * 
 * @author xtd
 * @date 2020-04-09
 */
@Service
public class DbTodoItemServiceImpl implements IDbTodoItemService 
{
    @Autowired
    private DbTodoItemMapper dbTodoItemMapper;

    /**
     * 查询待办事项
     * 
     * @param id 待办事项ID
     * @return 待办事项
     */
    @Override
    public DbTodoItem selectDbTodoItemById(Long id)
    {
        return dbTodoItemMapper.selectDbTodoItemById(id);
    }

    /**
     * 查询待办事项列表
     * 
     * @param dbTodoItem 待办事项
     * @return 待办事项
     */
    @Override
    public List<DbTodoItem> selectDbTodoItemList(DbTodoItem dbTodoItem)
    {
        return dbTodoItemMapper.selectDbTodoItemList(dbTodoItem);
    }

    /**
     * 新增待办事项
     * 
     * @param dbTodoItem 待办事项
     * @return 结果
     */
    @Override
    public int insertDbTodoItem(DbTodoItem dbTodoItem)
    {
        dbTodoItem.setCreateTime(DateUtils.getNowDate());
        return dbTodoItemMapper.insertDbTodoItem(dbTodoItem);
    }

    /**
     * 修改待办事项
     * 
     * @param dbTodoItem 待办事项
     * @return 结果
     */
    @Override
    public int updateDbTodoItem(DbTodoItem dbTodoItem)
    {
        dbTodoItem.setUpdateTime(DateUtils.getNowDate());
        return dbTodoItemMapper.updateDbTodoItem(dbTodoItem);
    }

    /**
     * 批量删除待办事项
     * 
     * @param ids 需要删除的待办事项ID
     * @return 结果
     */
    @Override
    public int deleteDbTodoItemByIds(Long[] ids)
    {
        return dbTodoItemMapper.deleteDbTodoItemByIds(ids);
    }

    /**
     * 删除待办事项信息
     * 
     * @param id 待办事项ID
     * @return 结果
     */
    @Override
    public int deleteDbTodoItemById(Long id)
    {
        return dbTodoItemMapper.deleteDbTodoItemById(id);
    }
}
